@@ -125,18 +125,6 @@ class _AbstractTransport(object):
                 if timeout != prev:
                     sock.settimeout(prev)
 
-    def __del__(self):
-        try:
-            # socket module may have been collected by gc
-            # if this is called by a thread at shutdown.
-            if socket is not None:
-                try:
-                    self.close()
-                except socket.error:
-                    pass
-        finally:
-            self.sock = None
-
     def _connect(self, host, port, timeout):
         entries = socket.getaddrinfo(
             host, port, 0, socket.SOCK_STREAM, SOL_TCP,
@@ -232,7 +220,7 @@ class _AbstractTransport(object):
         try:
             frame_header = read(7, True)
             read_frame_buffer += frame_header
-            frame_type, channel, size = unpack('>BHI', frame_header)
+            frame_type, channel, size = unpack(u'>BHI', frame_header)
             # >I is an unsigned int, but the argument to sock.recv is signed,
             # so we know the size can be at most 2 * SIGNED_INT_MAX
             if size > SIGNED_INT_MAX:
@@ -275,8 +263,7 @@ class SSLTransport(_AbstractTransport):
     """Transport that works over SSL"""
 
     def __init__(self, host, connect_timeout=None, ssl=None, **kwargs):
-        if isinstance(ssl, dict):
-            self.sslopts = ssl
+        self.sslopts = ssl if isinstance(ssl, dict) else {}
         self._read_buffer = EMPTY_BUFFER
         super(SSLTransport, self).__init__(
             host, connect_timeout=connect_timeout, **kwargs)
